unit SIXControlUnit;

{$mode objfpc}{$H+}

interface

uses
  Windows, Classes, SysUtils, Forms, Controls, Graphics, Crt,
  Dialogs, ExtCtrls, Buttons, ComCtrls, LazFileUtils,
  TAGraph, TASeries, TATools, TAChartUtils, TAFuncSeries,
  Types, TATextElements, TALegend, TACustomSeries, TAChartAxis;

type

  TSIXControl = class
    constructor create;
    procedure SCChartToolsetDataPointHintTool1Hint(ATool: TDataPointHintTool;
      const APoint: TPoint; var AHint: String);
    procedure SCChartToolsetTitleFootClickTool1Click(Sender: TChartTool;
      Title: TChartTitle);
    procedure SCChartToolsetLegendClickTool1Click(Sender: TChartTool;
      Legend: TChartLegend);
    procedure SCChartToolsetAxisClickTool1Click(Sender: TChartTool;
      AnAxis: TChartAxis; HitInfo: TChartAxisHitTests);

  private

  public

    class var
     evalTimeChanged: Boolean; // true if user changed evaluation time

  end;

var
  SIXControl: TSIXControl;
  InNameParse: string = ''; // name of load file
  DropfileNameParse: string = ''; // name of dropped file
  intSlopeCounter: Integer = 0; // counts how many time the current measurement exceeded the slope limit
  wasReset: Boolean = false; // true if Reset button was pressed
  ScreenOutName: string = '';
  CSVOutName: string = '';
  timeCounter : double = 0.0; // counter of the overall SIX signal time in min
  signalCounter : integer = 0; // counter of the overall SIX readouts
  Started: Boolean = false; //if Start was pressed or not

implementation

uses
  // ChartEditing units
  ceTitleFootDlg, ceLegendDlg, ceAxisDlg;

constructor TSIXControl.create;
begin
 evalTimeChanged:= false;
end;

procedure TSIXControl.SCChartToolsetDataPointHintTool1Hint(ATool: TDataPointHintTool;
  const APoint: TPoint; var AHint: String);
var
 x, y: Double;
begin
 with ATool as TDataPointHintTool do
  if (Series <> nil) then
   with (Series as TLineSeries) do
   begin
    x:= APoint.X; // only to avoid compiler warning about unused APoint
    y:= APoint.Y;
    // get the X/Y coordinate of the point
    x:= GetXValue(PointIndex);
    y:= GetYValue(PointIndex);
    AHint:= Format('x = %f' + LineEnding + 'y = %.3f', [x,y]);
   end;
end;

procedure TSIXControl.SCChartToolsetTitleFootClickTool1Click(Sender: TChartTool;
  Title: TChartTitle);
begin
 try
  TitleFootEditor.Prepare(Title, 'Edit chart title');
  TitleFootEditor.ShowModal;
 finally
  TitleFootEditor.Free;
 end;
end;

procedure TSIXControl.SCChartToolsetLegendClickTool1Click(Sender: TChartTool;
  Legend: TChartLegend);
begin
 try
  LegendEditor.Prepare(Legend, 'Edit chart legend');
  LegendEditor.ShowModal;
 finally
  LegendEditor.Free;
 end;
end;

procedure TSIXControl.SCChartToolsetAxisClickTool1Click(Sender: TChartTool;
  AnAxis: TChartAxis; HitInfo: TChartAxisHitTests);
var
  page: TAxisEditorPage;
begin
 Unused(Sender);
 if (ahtTitle in HitInfo) then
  page:= aepTitle
 else if (ahtLabels in HitInfo) then
  page:= aepLabels
 else if (ahtLine in HitInfo) then
  page:= aepLine
 else if (ahtGrid in HitInfo) then
  page:= aepGrid
 else
  exit;
 try
  AxisEditor.Prepare(AnAxis, 'Edit chart axis "%s"');
  AxisEditor.Page:= page;
  AxisEditor.ShowModal;
 finally
  AxisEditor.Free;
 end;
end;


end.

